//=============================================================================
//
// IBM WBI Brokers Replay
//
// Licensed Materials - Property of IBM Corp.
// 5648-C63 (c) Copyright IBM Corp. 2004
//
//=============================================================================

package samples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class TimeFormat
{
    private static final Pattern timeHMSPattern = Pattern.compile("^(?:(?:(\\d\\d?):)?(\\d\\d?):)?(\\d\\d?(?:\\.\\d+)?)$");
    private static final Pattern relativePattern = Pattern.compile("^(?i)(NOW|START|YESTERDAY|TODAY|TOMORROW)(?:\\s*([+-]|at)?\\s*(.+))?$");
    private static final SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy/MMM/dd HH:mm:ss.S");
    private static final SimpleDateFormat periodFormatter = new SimpleDateFormat("HH:mm:ss.S");

    // Parses input string of the form, 'NOW|START|YESTERDAY|TODAY|TOMORROW [[+/-|at] <period>]', 
    // e.g. 'NOW', 'NOW - 1:00', 'YESTERDAY at 9:30:00', 'TOMORROW at 15:59:59',
    // (or a java.util.Date).
    public static long timestampParse(String timestampString) throws IllegalArgumentException
    {
        long timestamp_ms = 0;

        Matcher relativeMatcher = relativePattern.matcher(timestampString);
        if(relativeMatcher.find())
        {
            String anchor = relativeMatcher.group(1);
            String sign = relativeMatcher.group(2);
            String periodString = relativeMatcher.group(3);

            Calendar cal = Calendar.getInstance();
            cal.clear(Calendar.HOUR);
            cal.clear(Calendar.AM_PM);
            cal.clear(Calendar.HOUR_OF_DAY);
            cal.clear(Calendar.MINUTE);
            cal.clear(Calendar.SECOND);
            cal.clear(Calendar.MILLISECOND);

            if(anchor.equalsIgnoreCase("NOW"))
            {
                timestamp_ms = System.currentTimeMillis();
            }
            else if(anchor.equalsIgnoreCase("START"))
            {
                timestamp_ms = 1;
            }
            else if(anchor.equalsIgnoreCase("YESTERDAY"))
            {
                cal.add(Calendar.DAY_OF_YEAR, -1);
                timestamp_ms = cal.getTime().getTime();
            }
            else if(anchor.equalsIgnoreCase("TODAY"))
            {
                timestamp_ms = cal.getTime().getTime();
            }
            else if(anchor.equalsIgnoreCase("TOMORROW"))
            {
                cal.add(Calendar.DAY_OF_YEAR, 1);
                timestamp_ms = cal.getTime().getTime();
            }

            if(null != periodString)
            {
                int multiplier = 1;
                if(null != sign  &&  sign.equals("-"))
                {
                    multiplier = -1;
                }

                timestamp_ms += multiplier * periodParse(periodString);
            }
        }
        else
        {
            timestamp_ms = Long.parseLong(timestampString);
        }

        return timestamp_ms;
    }



    // Parses input string of the form, '[[[H]H:}[m]m:][s...]s[.S[S...]]', 
    // e.g. '15:07:24.125', '15:7:24', '7:24', '24.125', '24'; 
    // returning the period in milliseconds, or throwing an IllegalArgumentException if unable to parse.
    public static long periodParse(String periodString) throws IllegalArgumentException
    {
        long period_ms = 0;

        Matcher timeHMSMatcher = timeHMSPattern.matcher(periodString);
        if(timeHMSMatcher.find())
        {
            String hh = timeHMSMatcher.group(1);
            String mm = timeHMSMatcher.group(2);
            String ss = timeHMSMatcher.group(3);

            int hours =   (null == hh) ? 0 : Integer.parseInt(hh);
            int minutes = (null == mm) ? 0 : Integer.parseInt(mm);
            float seconds = (null == ss) ? 0 : Float.parseFloat(ss);

            period_ms = (hours*1000*60*60) + (minutes*1000*60) + (Math.round(seconds*1000));
        }
        else
        {
            throw new IllegalArgumentException("Unable to parse '" + periodString + "' as a period.");
        }

        return period_ms;
    }


    public static String timestampFormat(long timestamp)
    {
        return dateFormatter.format(new Date(timestamp));
    }

    public static String periodFormat(long period)
    {
        return periodFormatter.format(new Date(period - periodFormatter.getTimeZone().getDSTSavings()));
    }

    public static void main(String[] args) throws IOException
    {
        String timestampString;
        long timestamp_ms = 0;
        BufferedReader commandLine = new java.io.BufferedReader(new InputStreamReader(System.in));

        while(true)
        {
            System.out.print("Enter timestamp attempt: ");
            if(0 == (timestampString = commandLine.readLine()).length()) break;

            try
            {
                timestamp_ms = timestampParse(timestampString);
                System.out.println("Parsed '" + timestampString + "' as timestamp, '" + timestamp_ms + "', (" + timestampFormat(timestamp_ms) + ").");
            }
            catch(IllegalArgumentException e)
            {
                System.out.println("Unable to parse '" + timestampString + "' as a timestamp.");

                try
                {
                    timestamp_ms = periodParse(timestampString);
                    System.out.println("Parsed '" + timestampString + "' as period, '" + timestamp_ms + "', (" + periodFormat(timestamp_ms) + ").");
                }
                catch(IllegalArgumentException e1)
                {
                    System.out.println("Unable to parse '" + timestampString + "'.");
                }
            }
        }
        System.out.println("Done.");
    }
}

